#include <arduino.h>
#include "Hardware_DLC35010R.h"
#include <ArduinoLog.h>

LGFX tft;                     //Global variable for display driver
//FT6236G ct;            // FT6236 touch driver variable. Access via I2C
TAMC_FT62X6 ct = TAMC_FT62X6();  // FT6236 touch driver variable. Access via I2C
PCA9548A i2cSelect;   // I2C multiplexer, modified base library mandatory !!
// LVGL variables
static lv_disp_draw_buf_t draw_buf;           // buffer for LVGL
static lv_color_t buf[ hres * vres / 10 ];    // buffer for LVGL
lv_obj_t * scr;                               // LVGL actual screen


void display_init()
{
  tft.begin();          // TFT init 
  tft.setRotation(0); // Portrait orientation
  tft.fillScreen(TFT_BLACK); // black background 
  tft.setBrightness(127);  // medium brightness
  tft.setTextColor(TFT_WHITE, TFT_BLACK);
  tft.setTextDatum(CC_DATUM);
  tft.drawString("TFT started", hres/2, vres/2, 4);
  delay(1000);
  tft.fillScreen(TFT_BLACK); // black background 
  Log.traceln("Display startet with LovyanGFX driver.");
}

// Display flushing from tft_eSPI or LovyanGFX to LVGL 
void my_disp_flush( lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p )
{
  uint32_t w = ( area->x2 - area->x1 + 1 );
  uint32_t h = ( area->y2 - area->y1 + 1 );

  tft.startWrite();
  tft.setAddrWindow( area->x1, area->y1, w, h );  // uncomment for LovyanGFX driver
  // tft.pushColors( ( uint16_t * )&color_p->full, w * h, true ); //uncomment for tft_eSPI
  tft.writePixels((lgfx::rgb565_t *)&color_p->full, w * h);
  tft.endWrite();

  lv_disp_flush_ready( disp );
}
/*
#if LV_USE_LOG != 0
// Serial debugging 
void my_print(const char * buf)
{
  Serial.printf(buf);
  Serial.flush();
}
#endif
*/

// Routine for reading the touchpad und send to lvgl
// Routine is specific for FT6236G library !!
void my_touchpad_read( lv_indev_drv_t * indev_drv, lv_indev_data_t * data )
{
  //TOUCHINFO ti;
  //Serial.println("touch request");
  ct.read();
  if (ct.isTouched && (ct.gesture == NO_GESTURE))
  {
    if (ct.touches)
    {
      //Set the coordinates
      data->point.x = ct.points[0].x;
      data->point.y = ct.points[0].y;
      data->state = LV_INDEV_STATE_PR;  
    }
    else 
    {
      data->state = LV_INDEV_STATE_REL;
    }  
  } else {
    data->state = LV_INDEV_STATE_REL;
  }
}

/*
void my_touchpad_read( lv_indev_drv_t * indev_drv, lv_indev_data_t * data )
{
  TOUCHINFO ti;
  //Serial.println("touch request");
  if (ct.getSamples(&ti) != FT_SUCCESS)
  {
    data->state = LV_INDEV_STATE_REL;
  } else {
    if (ti.count)
    {
      //Set the coordinates
      data->point.x = ti.x[0];
      data->point.y = ti.y[0];
      data->state = LV_INDEV_STATE_PR;  
    }
    else 
    {
      data->state = LV_INDEV_STATE_REL;
    }  
  }
}
*/

#if LV_USE_LOG != 0
void my_print(const char * buf);
#endif

void LVGL_begin()
{
  lv_init(); // start LVGL

  #if LV_USE_LOG != 0
    lv_log_register_print_cb( my_print ); // register print function for debugging
  #endif

  lv_disp_draw_buf_init( &draw_buf, buf, NULL, hres * vres / 10 );

  //Initialize the display
  static lv_disp_drv_t disp_drv;
  lv_disp_drv_init( &disp_drv );
  disp_drv.hor_res = hres;
  disp_drv.ver_res = vres;
  disp_drv.flush_cb = my_disp_flush; // allocation of data transfer routine
  disp_drv.draw_buf = &draw_buf;
  lv_disp_drv_register( &disp_drv );
  scr = lv_scr_act();            // default screen object -#



  //Initialize the (dummy) input device driver
  static lv_indev_drv_t indev_drv;
  lv_indev_drv_init( &indev_drv );
  indev_drv.type = LV_INDEV_TYPE_POINTER;
  indev_drv.read_cb = my_touchpad_read;
  //static lv_indev_t * my_touch = lv_indev_drv_register( &indev_drv );
  lv_indev_drv_register( &indev_drv );

}

void start_hardware()
{
  // initialise all hardware of WALPOL testbed

  Wire.setPins(PIN_IIC_SDA, PIN_IIC_SCL);
  Wire.begin();
  i2cSelect.attach(Wire, 0x70); // default addr : 0x70
  i2cSelect.enable(0); // switch PCA9548A to port 0
  Log.traceln("Port 0 selected ...");
  display_init();
  //ct.init(PIN_IIC_SDA, PIN_IIC_SCL, false, 100000); // start touch driver via I2C
  ct.begin();
  ct.setRotation(ROTATION_RIGHT);
  Log.traceln("Touch driver initialised.");
//  LVGL_begin();
//  Log.traceln("LVGL initialised.");
}
