#pragma once

#include <Arduino.h>
#define LGFX_USE_V1
#include <LovyanGFX.hpp>
#include <lvgl.h>
#include <TAMC_FT62X6.h>
#include "PCA9548.h"

/*ESP32S3*/
#define PIN_IIC_SDA                  38
#define PIN_IIC_SCL                  39

#define PIN_LCD_D0                   47
#define PIN_LCD_D1                   21
#define PIN_LCD_D2                   14
#define PIN_LCD_D3                   13
#define PIN_LCD_D4                   12
#define PIN_LCD_D5                   11
#define PIN_LCD_D6                   10
#define PIN_LCD_D7                   9
#define PIN_LCD_D8                   3
#define PIN_LCD_D9                   8
#define PIN_LCD_D10                  16
#define PIN_LCD_D11                  15
#define PIN_LCD_D12                  7
#define PIN_LCD_D13                  6
#define PIN_LCD_D14                  5
#define PIN_LCD_D15                  4

#define PIN_LCD_BL            46
#define PIN_LCD_RD            48
#define PIN_LCD_WR            18
#define PIN_LCD_DC            45
#define PIN_LCD_RES           -1

#define PIN_TP_INT            0
#define PIN_TP_RES            -1

#define PIN_BAT_VOLT          17

#define PIN_SD_CS             1
#define PIN_SD_SCK            42
#define PIN_SD_MOSI           2
#define PIN_SD_MISO           41

#define PIN_STEMMA_SDA        38
#define PIN_STEMMA_SCL        39

#define hres                  320   // physical screen resolution
#define vres                  480   // as above



class LGFX : public lgfx::LGFX_Device
{
    // lgfx::Panel_ILI9341 _panel_instance;
    lgfx::Panel_ILI9488 _panel_instance;
    lgfx::Bus_Parallel16 _bus_instance; // 16-bit parallel bus instance (ESP32-S3 only)
    lgfx::Light_PWM _light_instance;

public:
    // Create a constructor and configure various settings here.
    // If you change the class name, specify the same name for the constructor.
    LGFX(void)
    {
        {                                      // Configure bus control settings.
            auto cfg = _bus_instance.config(); // Get the bus configuration structure.

            // 16bit parallel setting
            cfg.port = 0;              // Select I2S port to use (0 or 1) (uses I2S LCD mode of ESP32)
            cfg.freq_write = 20000000; // Transmission clock (maximum 20MHz, rounded to 80MHz divided by an integer)
            cfg.pin_wr = PIN_LCD_WR;           // Pin number connecting WR
            cfg.pin_rd = PIN_LCD_RD;           // Pin number connecting RD
            cfg.pin_rs = PIN_LCD_DC;           // Pin number connecting RS(D/C)

            cfg.pin_d0 = PIN_LCD_D0;
            cfg.pin_d1 = PIN_LCD_D1;
            cfg.pin_d2 = PIN_LCD_D2;
            cfg.pin_d3 = PIN_LCD_D3;
            cfg.pin_d4 = PIN_LCD_D4;
            cfg.pin_d5 = PIN_LCD_D5;
            cfg.pin_d6 = PIN_LCD_D6;
            cfg.pin_d7 = PIN_LCD_D7;
            cfg.pin_d8 = PIN_LCD_D8;
            cfg.pin_d9 = PIN_LCD_D9;
            cfg.pin_d10 = PIN_LCD_D10;
            cfg.pin_d11 = PIN_LCD_D11;
            cfg.pin_d12 = PIN_LCD_D12;
            cfg.pin_d13 = PIN_LCD_D13;
            cfg.pin_d14 = PIN_LCD_D14;
            cfg.pin_d15 = PIN_LCD_D15;

            _bus_instance.config(cfg);              // Reflects the setting value on the bus.
            _panel_instance.setBus(&_bus_instance); // Place the bus on the panel.
        }

        {                                        // Configure display panel control settings.
            auto cfg = _panel_instance.config(); // Gets the structure for display panel settings.

            cfg.pin_cs = -1;   // CS required low
            cfg.pin_rst = -1;  // RST Japanese version board RST connection
            cfg.pin_busy = -1; // Pin number to which BUSY is connected (-1 = disable)

            // *The following setting values ​​are general default values ​​set for each panel, so please comment out any items you are unsure of and try again.

            cfg.memory_width = hres;   // Maximum width supported by driver IC
            cfg.memory_height = vres;  // Maximum height supported by driver IC
            cfg.panel_width = hres;    // Actual displayable width
            cfg.panel_height = vres;   // Actual display height
            cfg.offset_x = 0;         // Panel X direction offset amount
            cfg.offset_y = 0;         // Panel Y direction offset amount
            cfg.offset_rotation = 0;  // Offset of value in rotation direction 0~7 (4~7 are upside down)
            cfg.dummy_read_pixel = 8; // Number of bits for dummy read before pixel readout
            cfg.dummy_read_bits = 1;  // cfg.dummy_read_bits = 1; // Number of bits for dummy read before reading data other than pixels
            cfg.readable = true;      // Set to true if data reading is possible
            cfg.invert = false;       // Set to true if the brightness and darkness of the panel is reversed.
            cfg.rgb_order = false;    // Set to true if the red and blue colors of the panel are swapped.
            cfg.dlen_16bit = true;    // For panels that send data length in units of 16 bits Set to true
            cfg.bus_shared = false;    // Set to true when sharing the bus with the SD card (control the bus using drawJpgFile, etc.)

            _panel_instance.config(cfg);
        }

        { // Configure backlight control settings. (Delete if unnecessary)
            auto cfg = _light_instance.config();    // Gets the structure for backlight settings.

            cfg.pin_bl = PIN_LCD_BL;              // Pin number to which the backlight is connected
            cfg.invert = false;           // true to invert the backlight brightness
            cfg.freq   = 44100;           // Backlight PWM frequency
            cfg.pwm_channel = 7;          // PWM channel number to use

            _light_instance.config(cfg);
            _panel_instance.setLight(&_light_instance);  // Set the backlight on the panel.
    }

        setPanel(&_panel_instance); // Set the panel to be used.
    }
};

void start_hardware();
void my_disp_flush( lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p );
void my_touchpad_read( lv_indev_drv_t * indev_drv, lv_indev_data_t * data );
void LVGL_begin();